package com.koutra.nodejava.impl;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

public class HttpSession implements com.koutra.nodejava.HttpSession, Serializable {

	@Override
	public long getCreationTime() {
		if (!isValid()) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		return creationTime;
	}

	@Override
	public String getId() {
		return id;
	}

	@Override
	public long getLastAccessedTime() {
		if (!isValid()) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		return lastAccessedTime;
	}

	@Override
	public void setMaxInactiveInterval(int interval) {
		this.maxInactiveInterval = interval;
	}

	@Override
	public int getMaxInactiveInterval() {
		return maxInactiveInterval;
	}

	public boolean isValid() {
		if (maxInactiveInterval <= 0) {
			return true;
		}
		
		if (invalid) {
			return false;
		}
		
		Calendar creation = Calendar.getInstance();
		creation.setTimeInMillis(creationTime);
		Calendar now = Calendar.getInstance();
		now.setTime(new Date());
		creation.add(Calendar.SECOND, maxInactiveInterval);
		if (creation.before(now)) {
			// We must invalidate the session.
			invalidate();
			return false;
		}
		
		return true;
	}
	
	@Override
	public void invalidate() {
		if (invalid) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		this.invalid = true;
		this.attributeMap.clear();
	}

	@Override
	public boolean isNew() {
		if (!isValid()) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		return newFlag;
	}

	@Override
	public Object getAttribute(String name) {
		if (!isValid()) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		return attributeMap.get(name);
	}

	@Override
	public Set<String> getAttributeNames() {
		if (!isValid()) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		return attributeMap.keySet();
	}

	@Override
	public void setAttribute(String name, Object value) {
		if (!isValid()) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		attributeMap.put(name, value);
	}

	@Override
	public void removeAttribute(String name) {
		if (!isValid()) {
			throw new IllegalStateException("Session is invalid.");
		}
		
		attributeMap.remove(name);
	}

	////////////////////////////////////////////////////////////////////////////
	// End of interface implementation. Now for the specifics.
	////////////////////////////////////////////////////////////////////////////

	private static final long serialVersionUID = 335960189315157884L;

	protected boolean invalid;
	protected Map<String, Object> attributeMap;
	protected long creationTime;
	protected long lastAccessedTime;
	protected int maxInactiveInterval;
	protected boolean newFlag;
	protected String id;
	
	public HttpSession() {
		this.id = UUID.randomUUID().toString();
		this.newFlag = true;
		this.maxInactiveInterval = 0;
		this.creationTime = this.lastAccessedTime = new Date().getTime();
		this.attributeMap = new HashMap<String, Object>();
		this.invalid = false;
		SessionContainer.get().addSession(this);
	}
	
	public void resetNew() {
		this.newFlag = false;
	}
}
