package com.koutra.nodejava.impl;

import java.util.HashMap;
import java.util.Map;

public class SessionContainer {
	private static ThreadLocal<SessionContainer> sessionContainer =
			new ThreadLocal<SessionContainer>();
	
	/**
	 * Extreme caution is advised when using this method, as it relies on the
	 * fact that the stack trace has gone through the HttpProcessor implementation
	 * in the PatternRegistry class. This is generally true in the subsequent
	 * processors further down the stack, but the user is advised to make sure
	 * this is the case.
	 * @return
	 */
	public static SessionContainer get() {
		return sessionContainer.get();
	}
	
	public static void set(SessionContainer sessionContainer) {
		SessionContainer.sessionContainer.set(sessionContainer);
	}
	
	
	protected Map<String, HttpSession> sessionMap;
	
	public SessionContainer() {
		this.sessionMap = new HashMap<String, HttpSession>();
	}
	
	public HttpSession getSession(String id) {
		HttpSession retVal = sessionMap.get(id);
		if (retVal != null) {
			retVal.resetNew();
		}
		return retVal;
	}
	
	public void addSession(HttpSession session) {
		if (session == null) {
			throw new IllegalArgumentException(
					"Attempt to register a null session.");
		}
		sessionMap.put(session.getId(), session);
	}
	
	public void removeSession(String id) {
		sessionMap.remove(id);
	}
}
