package com.koutra.nodejava.utils;

import java.io.IOException;
import java.io.OutputStream;

/**
 * An OutputStream implementation that keeps a count of all the bytes that
 * are passed through the stream.
 * 
 * @author pftakas
 */
public class ByteCountingOutputStream extends OutputStream {
	
	protected int count;
	protected OutputStream underlyingOutputStream;
	
	public ByteCountingOutputStream(OutputStream underlyingOutputStream) {
		this.count = 0;
		this.underlyingOutputStream = underlyingOutputStream;
	}
	
	public int getByteCount() {
		return count;
	}

	@Override
	public void flush() throws IOException {
		underlyingOutputStream.flush();
	}
	
	@Override
	public void write(int b) throws IOException {
		count++;
		underlyingOutputStream.write(b);
	}
	
	@Override
	public void write(byte[] b) throws IOException {
		count += b.length;
		underlyingOutputStream.write(b);
	}
	
	@Override
	public void write(byte[] b, int off, int len) throws IOException {
		count += len;
		underlyingOutputStream.write(b, off, len);
	}

}
