package com.koutra.nodejava.utils;

import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A PrintWriter implementation that keeps a count of all the bytes that are
 * passed through the stream.
 * 
 * @author pftakas
 */
public class ByteCountingPrintWriter extends PrintWriter {
	private static final Logger logger = LoggerFactory.getLogger(
			ByteCountingPrintWriter.class);

	protected int count;
	protected int lineSeparatorByteCount;
	protected String charsetName;
	
	public ByteCountingPrintWriter(Writer writer, String charsetName) {
		super(writer);
		this.count = 0;
		this.charsetName = charsetName;
		
		String lineSeparator = System.getProperty("line.separator");
		try {
			this.lineSeparatorByteCount =
					lineSeparator.getBytes(charsetName).length;
		} catch (UnsupportedEncodingException e) {
			logger.error("Unable to count number of bytes.", e);
		}
	}
	
	public int getByteCount() {
		return count;
	}
	
	@Override
	public void write(int c) {
		try {
			count += new String(new char[] { ((char) c) }).getBytes(charsetName).length;
		} catch (UnsupportedEncodingException e) {
			logger.error("Unable to count number of bytes.", e);
		}
		super.write(c);
	}
	
	@Override
	public void write(char[] buf, int off, int len) {
		try {
			count += new String(buf, off, len).getBytes(charsetName).length;
		} catch (UnsupportedEncodingException e) {
			logger.error("Unable to count number of bytes.", e);
		}
		super.write(buf, off, len);
	}
	
	@Override
	public void write(String s, int off, int len) {
		try {
			if (off == 0 && len == s.length()) {
				count += s.getBytes(charsetName).length;
			} else {
				char[] cbuf = new char[len];
				s.getChars(off, (off + len), cbuf, 0);
				count += new String(cbuf, 0, len).getBytes(charsetName).length;
			}
		} catch (UnsupportedEncodingException e) {
			logger.error("Unable to count number of bytes.", e);
		}
		super.write(s, off, len);
	}
	
}
