package com.koutra.nodejava.utils;

import java.io.IOException;
import java.io.InputStream;

import com.koutra.nodejava.NodeJavaException;
import com.koutra.nodejava.ProcessClientTransportAgnostic;

public class HeaderReadingInputStream extends InputStream {
	private static final int CR = 0x0D;
	private static final int LF = 0x0A;

	private ProcessClientTransportAgnostic processClient;
	private InputStream socketInputStream;
	
	public HeaderReadingInputStream(ProcessClientTransportAgnostic processClient,
			InputStream socketInputStream) {
		this.processClient = processClient;
		this.socketInputStream = socketInputStream;
	}

	@Override
	public int read() throws IOException {
		return socketInputStream.read();
	}

	public String readHeaderLine(boolean notifyFirstCharTime) throws IOException, NodeJavaException {
		StringBuilder builder = new StringBuilder();
		boolean previousCharWasCR = false;
		boolean isFirstChar = true;
		while (true) {
			int charRead = socketInputStream.read();
			if (notifyFirstCharTime) {
				if (isFirstChar) {
					processClient.setStartTime(System.currentTimeMillis());
					isFirstChar = false;
				}
			}
			if (charRead == -1) {
				throw new NodeJavaException("Unable to read header line from " +
						"underlying input stream.");
			}
			if (charRead == LF && previousCharWasCR) {
				break;
			}
			previousCharWasCR = charRead == CR;
			builder.append((char) charRead);
		}
		return builder.toString().substring(0, builder.length() - 1);
	}
}
