package com.koutra.nodejava.velocity;

import org.apache.velocity.runtime.RuntimeServices;
import org.apache.velocity.runtime.log.LogChute;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class VelocityLogger implements LogChute {
	private static final Logger logger =
			LoggerFactory.getLogger(VelocityLogger.class);

	@Override
	public void init(RuntimeServices rs) throws Exception {
		// Nothing to do.
	}

	@Override
	public void log(int level, String message) {
		switch (level) {
		case LogChute.DEBUG_ID:
			logger.debug(message);
			break;
		case LogChute.ERROR_ID:
			logger.error(message);
			break;
		case LogChute.INFO_ID:
			logger.info(message);
			break;
		case LogChute.TRACE_ID:
			logger.trace(message);
			break;
		case LogChute.WARN_ID:
			logger.warn(message);
			break;
		default:
			throw new IllegalArgumentException("Unexpected log level: " + level);
		}
	}

	@Override
	public void log(int level, String message, Throwable t) {
		switch (level) {
		case LogChute.DEBUG_ID:
			logger.debug(message, t);
			break;
		case LogChute.ERROR_ID:
			logger.error(message, t);
			break;
		case LogChute.INFO_ID:
			logger.info(message, t);
			break;
		case LogChute.TRACE_ID:
			logger.trace(message, t);
			break;
		case LogChute.WARN_ID:
			logger.warn(message, t);
			break;
		default:
			throw new IllegalArgumentException("Unexpected log level: " + level);
		}
	}

	@Override
	public boolean isLevelEnabled(int level) {
		switch (level) {
		case LogChute.DEBUG_ID:
			return logger.isDebugEnabled();
		case LogChute.ERROR_ID:
			return logger.isErrorEnabled();
		case LogChute.INFO_ID:
			return logger.isInfoEnabled();
		case LogChute.TRACE_ID:
			return logger.isTraceEnabled();
		case LogChute.WARN_ID:
			return logger.isWarnEnabled();
		default:
			throw new IllegalArgumentException("Unexpected log level: " + level);
		}
	}

}
