package com.koutra.nodejava.velocity;

import java.io.IOException;
import java.util.Properties;

import org.apache.velocity.app.Velocity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.koutra.nodejava.HttpRequest;
import com.koutra.nodejava.HttpResponse;
import com.koutra.nodejava.NodeJava;
import com.koutra.nodejava.NodeJavaException;
import com.koutra.nodejava.StatelessHttpProcessor;

public class VelocityProcessor extends StatelessHttpProcessor {
	private static final Logger logger = LoggerFactory.getLogger(VelocityProcessor.class);
	
	private static final String VELOCITY_CONTEXT_FACILITY = "VelocityContextFacility";
	
	static {
	    Velocity.setProperty(
	    		Velocity.RUNTIME_LOG_LOGSYSTEM, new VelocityLogger());
	    try {
		    Properties velocityProperties = new Properties();
			velocityProperties.load(VelocityProcessor.class.getResourceAsStream(
					"/velocity.properties"));
		    Velocity.init(velocityProperties);
		} catch (IOException e) {
			logger.error("Unable to initialize velocity processor.", e);
			throw new IllegalStateException(
					"Unable to initialize velocity processor.", e);
		}
	}
	
	public static String convertPathToClassName(String path) {
		// Drop the leading '/' from the resource path
		// Drop the trailing '.vm' from the resource path
		// Replace any '/' characters in between the path elements with '.'
		// Thus, we get the class name of the velocity context
		String contextClassName = path.substring(
				1, path.lastIndexOf('.')).replaceAll("/", ".");
		return contextClassName;
	}
	
	@Override
	public void init(NodeJava nodeJava) throws NodeJavaException {
		super.init(nodeJava);
		nodeJava.registerFacility(VELOCITY_CONTEXT_FACILITY, new VelocityContextFacility());
	}
	
	protected void processUnknownResource(HttpRequest request, HttpResponse response)
					throws NodeJavaException {
		try {
			VelocityContextFacility fac = (VelocityContextFacility)
					nodeJava.retrieveFacility(VELOCITY_CONTEXT_FACILITY);
			NodeJavaVelocityContext vc = fac.getNodeJavaVelocityContext(
					UnknownResource.class.getName(),
					"/com/koutra/nodejava/velocity/UnknownResource.vm");
			vc.process(request, response);
		} catch (Exception e) {
			logger.error("Unable to run velocity tempalte.", e);
			throw new NodeJavaException("Unable to run velocity tempalte.", e);
		}
	}

	@Override
	public void process(HttpRequest request, HttpResponse response)
			throws NodeJavaException {
		VelocityContextFacility fac = (VelocityContextFacility)
				nodeJava.retrieveFacility(VELOCITY_CONTEXT_FACILITY);

		String requestPath = request.getPath();
		if (!fac.hasRequestPathBeenVisited(requestPath)) {
			if (!Velocity.resourceExists(requestPath)  ||
					requestPath.endsWith("/")) {
				processUnknownResource(request, response);
				return;
			}
		}
		try {
			String contextClassName = convertPathToClassName(requestPath);
			NodeJavaVelocityContext vc = fac.getNodeJavaVelocityContext(
					contextClassName, requestPath);
			vc.process(request, response);
		} catch (Exception e) {
			logger.error("Unable to run velocity tempalte.", e);
			throw new NodeJavaException("Unable to run velocity tempalte.", e);
		}
	}

}
