package com.koutra.nodejava;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.xml.sax.SAXException;

import com.koutra.nodejava.impl.HttpRequest;
import com.koutra.nodejava.impl.HttpResponse;
import com.koutra.nodejava.utils.ChunkedEncodingVerifyingOutputStream;
import com.koutra.nodejava.utils.HttpConstants;
import com.koutra.nodejava.utils.ReaderInputStream;
import com.koutra.nodejava.utils.WriterOutputStream;

public class ChunkedEncodingTest implements TimeRecorder {
	private static final Logger logger = Logger.getLogger(ChunkedEncodingTest.class);

	@Test
	public void test() throws ParserConfigurationException, SAXException, IOException, ClassNotFoundException, InstantiationException, IllegalAccessException, NodeJavaException {
		InputStream inputStream =
				new ReaderInputStream(new StringReader(RequestForAPage), "UTF-8");
		StringWriter stringWriter = new StringWriter();
		OutputStream outputStream =
				new WriterOutputStream(stringWriter, "UTF-8");
		ProcessClientTransportAgnostic processClient =
				new ProcessClientTransportAgnostic();
		processClient.init(new NodeJava(null), this, false, REMOTE_ADDR,
						REMOTE_HOST, SERVER_NAME, SERVER_PORT, inputStream,
						outputStream);
		HttpRequest request = processClient.processHeaders();
		ChunkedEncodingVerifyingOutputStream result =
				new ChunkedEncodingVerifyingOutputStream();
		HttpResponse response = new HttpResponse(result);
		response.setChunkedEncoding(true);
		response.setBufferSize(BUFFER_SIZE);
		
		processClient.processRequest(request, response);
		response.terminate();
		
		String responseString = result.getOutputString();
		
		int endOfHeadersIndex = responseString.indexOf(
				HttpConstants.CRLF + HttpConstants.CRLF);
		
		assertTrue(endOfHeadersIndex > 0);
		
		String headers = responseString.substring(0, endOfHeadersIndex);
		logger.info("Headers read: '" + headers + "'.");
		String remains = responseString.substring(endOfHeadersIndex + 4);
		logger.info("Remainder read: '" + remains + "'.");

		int totalBodyBytes = 0;
		while (true) {
			int byteCountEndIndex = remains.indexOf(HttpConstants.CRLF);
			assertTrue(byteCountEndIndex > 0);
			String byteCountString = remains.substring(0, byteCountEndIndex);
			remains = remains.substring(byteCountEndIndex + 2);
			
			logger.info("Byte Count is: " + byteCountString);
			
			if (byteCountString.equals("0")) {
				// We have reached the end of the response.
				break;
			}
			
			int bytesToRead = Integer.parseInt(byteCountString, 16);
			totalBodyBytes += bytesToRead;
			
			String part = remains.substring(0, bytesToRead);
			remains = remains.substring(bytesToRead);
			
			logger.info("Read part: '" + part + "'.");
			
			assertTrue(remains.startsWith(HttpConstants.CRLF));
			remains = remains.substring(2);
		}
		
		assertTrue(totalBodyBytes == 157);
	}

	@Override
	public void setStartTime(long startTime) {
		// Ignore this.
	}

	private static final String RequestForAPage =
			"GET /com/koutra/nodejava/TestPage.vm HTTP/1.1\r\n" +
			"Host: localhost:8080\r\n" +
			"User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3\r\n" +
			"Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\r\n" +
			"Accept-Language: en-us\r\n" +
			"Accept-Encoding: gzip, deflate\r\n" +
			"Connection: keep-alive\r\n" +
			"\r\n";
	
	private static final String REMOTE_ADDR = "REMOTE_ADDR";
	private static final String REMOTE_HOST = "REMOTE_HOST";
	private static final String SERVER_NAME = "SERVER_NAME";
	private static final int SERVER_PORT = 111;
	private static final int BUFFER_SIZE = 20;
}
