package com.koutra.nodejava;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.ParserConfigurationException;

import org.junit.Test;
import org.xml.sax.SAXException;

import com.koutra.nodejava.utils.HttpConstants;
import com.koutra.nodejava.utils.ReaderInputStream;
import com.koutra.nodejava.utils.WriterOutputStream;

public class ProcessClientTest implements TimeRecorder {

	@Test
	public void testRequestForAPage() throws ParserConfigurationException,
			SAXException, IOException, ClassNotFoundException,
			NodeJavaException, InstantiationException, IllegalAccessException {
		InputStream inputStream =
				new ReaderInputStream(new StringReader(RequestForAPage), "UTF-8");
		StringWriter stringWriter = new StringWriter();
		OutputStream outputStream =
				new WriterOutputStream(stringWriter, "UTF-8");
		ProcessClientTransportAgnostic processClient =
				new ProcessClientTransportAgnostic();
		processClient.init(new NodeJava(null), this, false, REMOTE_ADDR,
						REMOTE_HOST, SERVER_NAME, SERVER_PORT, inputStream,
						outputStream);
		HttpRequest request = processClient.processHeaders();
		
		assertTrue(SERVER_NAME.equals(request.getServerName()));
		assertTrue(SERVER_PORT == request.getServerPort());
		assertTrue(REMOTE_ADDR.equals(request.getRemoteAddr()));
		assertTrue(REMOTE_HOST.equals(request.getRemoteHost()));
		assertTrue("/com/koutra/nodejava/TestPage.vm".equals(request.getPath()));
		assertEquals(0, request.getParameterMap().size());
		assertEquals(6, request.getHeaderNames().size());
		assertEquals("localhost:8080",
				request.getHeader(HttpConstants.HEADER_NAME_HOST));
		assertEquals("Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) " +
				"AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 " +
				"Safari/534.48.3",
				request.getHeader(HttpConstants.HEADER_NAME_USER_AGENT));
		assertEquals("text/html,application/xhtml+xml,application/xml;" +
				"q=0.9,*/*;q=0.8",
				request.getHeader(HttpConstants.HEADER_NAME_ACCEPT));
		assertEquals("en-us",
				request.getHeader(HttpConstants.HEADER_NAME_ACCEPT_LANGUAGE));
		assertEquals("gzip, deflate",
				request.getHeader(HttpConstants.HEADER_NAME_ACCEPT_ENCODING));
		assertEquals(HttpConstants.HEADER_VALUE_KEEP_ALIVE,
				request.getHeader(HttpConstants.HEADER_NAME_CONNECTION));
	}

	@Override
	public void setStartTime(long startTime) {
		// Ignore this.
	}

	private static final String RequestForAPage =
			"GET /com/koutra/nodejava/TestPage.vm HTTP/1.1\r\n" +
			"Host: localhost:8080\r\n" +
			"User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3\r\n" +
			"Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\r\n" +
			"Accept-Language: en-us\r\n" +
			"Accept-Encoding: gzip, deflate\r\n" +
			"Connection: keep-alive\r\n" +
			"\r\n";
	
	private static final String REMOTE_ADDR = "REMOTE_ADDR";
	private static final String REMOTE_HOST = "REMOTE_HOST";
	private static final String SERVER_NAME = "SERVER_NAME";
	private static final int SERVER_PORT = 111;
}
