package com.koutra.nodejava;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;

public class StressTest {
	private static final Logger logger = Logger.getLogger(StressTest.class);
	
	private static final int CR = 0x0D;
	private static final int LF = 0x0A;

// This is really a unit test, but it requires that the SimpleNodeJava main
// method is executing on some other VM on the same machine. So commenting
// out the unit test annotation.
//	@Test
	public void test() throws UnknownHostException, IOException {
		long startTime = System.currentTimeMillis();
		
		Socket socket = new Socket("localhost", 8080);
		InputStream is = new BufferedInputStream(socket.getInputStream());
		OutputStream os = socket.getOutputStream();
		
		for (int i = 0; i < 100000; i++) {
			socketWillClose = false;
			
			String response = performRequest(is, os);
			if (logger.isTraceEnabled()) {
				logger.trace("Received response:\n" + response);
			}
			
			if (socketWillClose) {
				socket.close();
				socket = new Socket("localhost", 8080);
				is = new BufferedInputStream(socket.getInputStream());
				os = socket.getOutputStream();
			}
		}
		
		long endTime = System.currentTimeMillis();
		
		logger.info("Total time: " + ((endTime - startTime) / 1000.0) + "sec.");
	}
	
	private static byte[] buffer = new byte[8 * 1024];
	private static boolean socketWillClose;
	
	private static String performRequest(InputStream is, OutputStream os) throws IOException {
		os.write(RequestForAPage.getBytes());
		os.flush();
		int byteContent = readHeaders(is);
		if (byteContent > 0) {
			is.read(buffer, 0, byteContent);
		}
		return new String(buffer, 0, byteContent);
	}
	
	private static int readHeaders(InputStream is) throws IOException {
		int contentLength = 0;
		while (true) {
			String headerLine = readLine(is);
			if (logger.isTraceEnabled()) {
				logger.trace("INCOMING: " + headerLine);
			}
			if (headerLine.startsWith("Content-Length: ")) {
				String numberOfBytes = headerLine.substring("Content-Length: ".length());
				contentLength = Integer.parseInt(numberOfBytes);
			}
			if (headerLine.equals("Connection: close")) {
				socketWillClose = true;
			}
			if (headerLine.length() == 0) {
				// We have reached the end of the headers.
				break;
			}
		}
		return contentLength;
	}
	
	private static String readLine(InputStream is) throws IOException {
		StringBuilder sb = new StringBuilder();
		boolean previousCharWasCR = false;
		while (true) {
			int charRead = is.read();
			if (charRead == -1) {
				// Unexpected. Throw a fit!
				throw new IOException("Unexpected end of stream.");
			}
			if (charRead == LF && previousCharWasCR) {
				break;
			}
			previousCharWasCR = charRead == CR;
			sb.append((char) charRead);
		}
		return sb.toString().substring(0, sb.length() - 1);
	}

	private static final String RequestForAPage =
			"GET /foofootos/Test.vm HTTP/1.1\r\n" +
			"Host: localhost:8080\r\n" +
			"User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3\r\n" +
			"Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\r\n" +
			"Accept-Language: en-us\r\n" +
			"Accept-Encoding: gzip, deflate\r\n" +
			"Connection: keep-alive\r\n" +
			"\r\n";
}
