package com.koutra.nodejava.utils;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;

public class ReaderInputStream extends InputStream {
	
	private Reader reader;
	private String encoding;
	private byte[] remainsBuffer;
	private int startIndex;
	
	public ReaderInputStream(Reader reader, String encoding) {
		this.reader = reader;
		this.encoding = encoding;
		this.remainsBuffer = null;
		this.startIndex = 0;
	}

	@Override
	public int read() throws IOException {
        byte retVal;
        if (remainsBuffer != null && startIndex < remainsBuffer.length) {
        	retVal = remainsBuffer[startIndex];
            if (++startIndex == remainsBuffer.length) {
                remainsBuffer = null;
            }
        } else {
            byte[] buffer = new byte[1];
            if (read(buffer, 0, 1) <= 0) {
            	retVal = -1;
            }
            retVal = buffer[0];
        }
        
        retVal = (byte) (retVal < -1 ? retVal + 256 : retVal);

        return retVal;
	}
	
	@Override
	public int read(byte[] b, int off, int len) throws IOException {
        while (remainsBuffer == null) {
            char[] buffer = new char[len];
            int n = reader.read(buffer);
            if (n == -1) {
                return -1;
            }
            if (n > 0) {
                remainsBuffer = new String(buffer, 0, n).getBytes(encoding);
                startIndex = 0;
            }
        }
        
        len = len > remainsBuffer.length - startIndex ? remainsBuffer.length - startIndex : len;

        System.arraycopy(remainsBuffer, startIndex, b, off, len);

        startIndex += len;
        
        remainsBuffer = startIndex >= remainsBuffer.length ? null : remainsBuffer;

        return len;
	}
	
	@Override
	public void close() throws IOException {
		reader.close();
	}

}
